﻿'A reference to the Phidget22.NET.dll must be added to new projects in VB.Net 
'From the "Project" menu select "Add Reference" 
'From the "Add Reference" window select the "Browse" tab 
'Browse to the directory you installed the Phidgets library to 
'The default is "C:\Program Files\Phidgets" 
'Select "Phidget22.NET.dll" and click "OK" 
'To view objects that have been added, select the "View" menu 
'Select "Object Browser" 
'Then expand the Phidget22.NET.dll item 



Imports Phidget22
Imports Phidget22.Events
Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.Threading

Public Class Form1
    Dim WithEvents device As Phidget22.LCD
    Dim open As ExampleUtils.ExampleUtils.CommandLineOpen
    Dim errorBox As ErrorEventBox

    Dim LCD_WIDTH_PIXELS As Integer = 128
    Dim LCD_HEIGHT_PIXELS As Integer = 64
    Dim sleepCNT As Integer

    Public Sub New()
        open = New ExampleUtils.ExampleUtils.CommandLineOpen(Me)
        ' This call is required by the Windows Form Designer.
        Phidget22.Phidget.InvokeEventCallbacks = True
        InitializeComponent()
        ' Add any initialization after the InitializeComponent() call.
    End Sub

    Public Sub New(ByVal commandLine As String())
        open = New ExampleUtils.ExampleUtils.CommandLineOpen(Me)
        open.commandLine = commandLine
        Phidget22.Phidget.InvokeEventCallbacks = True
        InitializeComponent()
    End Sub

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        errorBox = New ErrorEventBox(Me)
        device = New Phidget22.LCD()
        open.openCmdLine(device)
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If (writeAnimatedGifThread IsNot Nothing And writeAnimatedGifThread.IsAlive) Then
            stop_writeAnimatedGifThread = True
            writeAnimatedGifThread.Join()
        End If

        If (asyncTextThread IsNot Nothing) Then
            asyncTextThreadRun = False
            asyncTextThread.Join()
            asyncTextThread = Nothing
        End If

        timer2.Stop()

        Application.DoEvents()
        If device.Attached Then
            device.Close()
        End If
    End Sub

    Private Sub lcd_Attach(ByVal sender As Object, ByVal e As Phidget22.Events.AttachEventArgs) Handles device.Attach
        Dim attachedDevice As Phidget22.LCD = CType(sender, Phidget22.LCD)
        PhidgetInfoBox1.FillPhidgetInfo(attachedDevice)

        Try
            device.Sleeping = False

            'these are just generic defaults, they can be changed to suit your application
            device.Backlight = 0.5 '50% backlight
            device.Contrast = 0.25 '25% contrast
            contrasttrk.Value = CType(device.Contrast * 255.0, Integer)
            brightnesstrk.Value = CType(device.Backlight * 255.0, Integer)

            'load whatever is stored in flash memory
            device.FrameBuffer = 2
            device.Flush()

            LCD_WIDTH_PIXELS = device.Width
            LCD_HEIGHT_PIXELS = device.Height
        Catch ex As Exception
            errorBox.addMessage("Error initializing device: " + ex.Message)
        End Try

        settingsBox.Visible = True
    End Sub

    Private Sub lcd_Detach(ByVal sender As Object, ByVal e As Phidget22.Events.DetachEventArgs) Handles device.Detach
        timer2.Stop()
        PhidgetInfoBox1.Clear()
        settingsBox.Visible = False
    End Sub

    Private Sub lcd_Error(ByVal sender As Object, ByVal e As Phidget22.Events.ErrorEventArgs) Handles device.Error
        errorBox.addMessage(e.Description)
    End Sub

    Dim rand As Random = New Random()
    Dim asyncTextThread As System.Threading.Thread
    Dim asyncTextThreadRun As Boolean

    Private Sub asyncTextThreadFunction()
        While asyncTextThreadRun
            Dim x As Integer = rand.Next(127)
            Dim y As Integer = -rand.Next(63)

            Dim res1 As IAsyncResult = device.BeginWriteText(LCDFont.Dimensions_5x8, x, y, rand.NextDouble().ToString("F2"), Nothing, Nothing)
            Dim res2 As IAsyncResult = device.BeginFlush(Nothing, Nothing)

            Try
                device.EndWriteText(res1)
            Catch ex As Exception
                errorBox.addMessage("Async Error writing text: " + ex.Message)
            End Try

            Try
                device.EndFlush(res2)
            Catch ex As Exception
                errorBox.addMessage("Async Error flushing: " + ex.Message)
            End Try
        End While
    End Sub

    Public Sub ClearAsyncCallback(ByVal result As IAsyncResult)
        Try
            device.EndClear(result)
        Catch ex As Exception
            errorBox.addMessage("Async Error on Clear: " + ex.Message)
        End Try
    End Sub

    Public Sub WriteBitmapAsyncCallback(ByVal result As IAsyncResult)
        Try
            device.EndWriteBitmap(result)
        Catch ex As Exception
            errorBox.addMessage("Async Error on WriteBitmap: " + ex.Message)
        End Try
    End Sub

    Public Sub FlushAsyncCallback(ByVal result As IAsyncResult)
        Try
            device.EndFlush(result)
        Catch ex As Exception
            errorBox.addMessage("Async Error on Flush: " + ex.Message)
        End Try
    End Sub

    Private Sub writeImage(ByVal pic As Bitmap)
        'resize to fit within lcd width and height
        Dim width As Integer = pic.Width
        Dim height As Integer = pic.Height
        If (width > device.Width) Then
            Dim factor As Double = CType(device.Width, Double) / CType(width, Double)
            width = CType(width * factor, Integer)
            height = CType(height * factor, Integer)
        End If
        If (height > device.Height) Then
            Dim factor As Double = CType(device.Height, Double) / CType(height, Double)
            width = CType(width * factor, Integer)
            height = CType(height * factor, Integer)
        End If

        Dim pic2 As Bitmap = New Bitmap(pic, width, height)

        'conver to 1-bit and copy to byte array
        Dim imageForLCD(width * height) As Byte
        For y As Integer = 0 To height - 1
            For x As Integer = 0 To width - 1
                Dim c As Color = pic2.GetPixel(x, y)
                If (c.GetBrightness() >= 0.7) Then
                    imageForLCD(x + y * width) = 0
                Else
                    imageForLCD(x + y * width) = 1
                End If
            Next
        Next

        pictureBox1.Image = pic2

        device.BeginClear(New AsyncCallback(AddressOf ClearAsyncCallback), Nothing)
        device.BeginWriteBitmap(0, 0, width, height, imageForLCD, New AsyncCallback(AddressOf WriteBitmapAsyncCallback), Nothing)
        device.BeginFlush(New AsyncCallback(AddressOf FlushAsyncCallback), Nothing)
    End Sub

    Private Sub writeImageGreyscale(ByVal pic As Bitmap)
        'resize to fit within lcd width and height
        Dim width As Integer = pic.Width
        Dim height As Integer = pic.Height
        If width > device.Width Then
            Dim factor = CType(device.Width, Double) / CType(width, Double)
            width = CType(width * factor, Integer)
            height = CType(height * factor, Integer)
        End If
        If height > device.Height Then
            Dim factor = CType(device.Height, Double) / CType(height, Double)
            width = CType(width * factor, Integer)
            height = CType(height * factor, Integer)
        End If

        Dim pic2 As Bitmap = New Bitmap(pic, width, height)

        'convert to 1-bit and copy to byte array
        Dim imageForLCD0(width * height) As Byte
        Dim imageForLCD1(width * height) As Byte
        Dim imageForLCD2(width * height) As Byte

        Dim picForView As Bitmap = New Bitmap(width, height)

        For y As Integer = 0 To width - 1
            For x As Integer = 0 To width - 1
                'all to black
                imageForLCD0(x + y * width) = 1
                imageForLCD1(x + y * width) = 1
                imageForLCD2(x + y * width) = 1
                'set to white as brighter
                Dim c As Color = pic2.GetPixel(x, y)
                picForView.SetPixel(x, y, Color.Black)
                If c.GetBrightness() >= 0.2 Then
                    imageForLCD0(x + y * width) = 0
                    picForView.SetPixel(x, y, Color.DarkGray)
                End If
                If c.GetBrightness() >= 0.5 Then
                    imageForLCD1(x + y * width) = 0
                    picForView.SetPixel(x, y, Color.LightGray)
                End If
                If c.GetBrightness() >= 0.8 Then
                    imageForLCD2(x + y * width) = 0
                    picForView.SetPixel(x, y, Color.White)
                End If
            Next
        Next

        pictureBox1.Image = picForView

        device.FrameBuffer = 0
        device.BeginClear(New AsyncCallback(AddressOf ClearAsyncCallback), Nothing)
        device.BeginWriteBitmap(0, 0, width, height, imageForLCD0, New AsyncCallback(AddressOf WriteBitmapAsyncCallback), Nothing)
        device.BeginFlush(New AsyncCallback(AddressOf FlushAsyncCallback), Nothing)

        device.FrameBuffer = 1
        device.BeginClear(New AsyncCallback(AddressOf ClearAsyncCallback), Nothing)
        device.BeginWriteBitmap(0, 0, width, height, imageForLCD1, New AsyncCallback(AddressOf WriteBitmapAsyncCallback), Nothing)
        device.BeginFlush(New AsyncCallback(AddressOf FlushAsyncCallback), Nothing)

        device.FrameBuffer = 2
        device.BeginClear(New AsyncCallback(AddressOf ClearAsyncCallback), Nothing)
        device.BeginWriteBitmap(0, 0, width, height, imageForLCD2, New AsyncCallback(AddressOf WriteBitmapAsyncCallback), Nothing)
        device.BeginFlush(New AsyncCallback(AddressOf FlushAsyncCallback), Nothing)

        timer2.Start()

    End Sub


    Dim image As Image
    Dim writeAnimatedGifThread As System.Threading.Thread
    Dim stop_writeAnimatedGifThread As Boolean = False
    Dim fps As Double = 0
    Delegate Sub InvokeDelegate()

    Sub writeAnimatedGifThreadFunction()
        Dim repeat As Integer = 2
        Dim numberOfFrames As Integer = image.GetFrameCount(FrameDimension.Time)

        While repeat > 0
            repeat -= 1
            For i As Integer = 0 To numberOfFrames - 1
                Dim sw As Stopwatch = New Stopwatch()
                sw.Start()

                image.SelectActiveFrame(FrameDimension.Time, i)
                Dim pic As Bitmap = New Bitmap(image)

                Try
                    writeImage(pic)
                Catch ex As Exception
                    errorBox.addMessage("Error writing image: " + ex.Message)
                    Return
                End Try
                fps = 1 / (sw.ElapsedMilliseconds / 1000.0)
                fpsLbl.BeginInvoke(New InvokeDelegate(AddressOf fpsInvoker))

                If (stop_writeAnimatedGifThread) Then
                    Return
                End If
            Next
        End While
    End Sub

    Public Sub fpsInvoker()
        fpsLbl.Text = fps.ToString("F3" + " fps")
    End Sub

    Private Sub timer2_Tick(sender As Object, e As EventArgs) Handles timer2.Tick
        If device.FrameBuffer = 0 Then
            device.BeginSetFrameBuffer(1, Nothing, Nothing)
        ElseIf device.FrameBuffer = 1
            device.BeginSetFrameBuffer(2, Nothing, Nothing)
        Else
            device.BeginSetFrameBuffer(0, Nothing, Nothing)
        End If
        device.BeginFlush(Nothing, Nothing)
    End Sub

    Private Sub contrasttrk_Scroll(sender As Object, e As EventArgs) Handles contrasttrk.Scroll
        Try
            device.Contrast = CType(contrasttrk.Value, Double) / 255.0
        Catch ex As Exception
            errorBox.addMessage("Error setting contrast: " + ex.Message)
        End Try
    End Sub

    Private Sub brightnesstrk_Scroll(sender As Object, e As EventArgs) Handles brightnesstrk.Scroll
        Try
            device.Backlight = CType(brightnesstrk.Value, Double) / 255.0
        Catch ex As Exception
            errorBox.addMessage("Error setting brightness: " + ex.Message)
        End Try
    End Sub

    Public Sub DrawLineAsyncCallback(ByVal result As IAsyncResult)
        Try
            device.EndDrawLine(result)
        Catch ex As Exception
            errorBox.addMessage("Async Error drawing line: " + ex.Message)
        End Try
    End Sub

    Public Sub DrawRectAsyncCallback(ByVal result As IAsyncResult)
        Try
            device.EndDrawRectangle(result)
        Catch ex As Exception
            errorBox.addMessage("Async Error drawing rect: " + ex.Message)
        End Try
    End Sub

    Public Sub CopyAsyncCallback(ByVal result As IAsyncResult)
        Try
            device.EndCopy(result)
        Catch ex As Exception
            errorBox.addMessage("Async Error copying: " + ex.Message)
        End Try
    End Sub

    Private Sub rectSamplebtn_Click(sender As Object, e As EventArgs) Handles rectSamplebtn.Click
        timer2.Stop()
        Try
            'select frame buffer to draw to
            device.FrameBuffer = 0
            device.Clear()

            'draw some stuff asynchronously
            For i As Integer = 0 To 7
                device.BeginDrawLine(0, (i * 8), (LCD_WIDTH_PIXELS - 1), (i * 8), New AsyncCallback(AddressOf DrawLineAsyncCallback), Nothing)
            Next
            For i As Integer = 0 To 20
                device.BeginDrawRectangle((i * 6), 0, ((i * 6) + 6), 7, True, If((i Mod 2) = 0, False, True), New AsyncCallback(AddressOf DrawRectAsyncCallback), Nothing)
            Next

            device.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 9, False, New AsyncCallback(AddressOf CopyAsyncCallback), Nothing)
            device.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 4, 6, 4, 17, False, New AsyncCallback(AddressOf CopyAsyncCallback), Nothing)
            device.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 25, False, New AsyncCallback(AddressOf CopyAsyncCallback), Nothing)
            device.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1), 6, 0, 33, False, New AsyncCallback(AddressOf CopyAsyncCallback), Nothing)
            device.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 41, False, New AsyncCallback(AddressOf CopyAsyncCallback), Nothing)
            device.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 4, 6, 4, 49, False, New AsyncCallback(AddressOf CopyAsyncCallback), Nothing)
            device.BeginCopy(0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 57, False, New AsyncCallback(AddressOf CopyAsyncCallback), Nothing)

            'write the frame buffer to the screen
            device.Flush()
        Catch ex As Exception
            errorBox.addMessage("Error drawing image: " + ex.Message)
        End Try
    End Sub

    Private Function getBitmapBytes(ByVal charStr As String) As Byte()
        Dim bytes(charStr.Length / 2) As Byte
        For i As Integer = 0 To charStr.Length / 2 - 1
            bytes(i) = CType(If(charStr.ToCharArray()(i * 2) = " ", &H00, &H01), Byte)
        Next
        Return bytes
    End Function

    Private Sub basicExamplebtn_Click(sender As Object, e As EventArgs) Handles basicExamplebtn.Click
        timer2.Stop()
        Dim aBitmap As Byte()
        Dim csBitmap As Byte()

        Try
            'select the frame buffer to draw to
            device.FrameBuffer = 0
            device.Clear()

#Region "bitmapDefinitions"
            'Note: I'm using 2 character per bit strings to increase readability in the code.  Actual functions require byte arrays.
            aBitmap = getBitmapBytes(
                      "      ██████      " +
                      "    ██████████    " +
                      "  ████      ████  " +
                      "  ████      ████  " +
                      "  ██████████████  " +
                      "  ██████████████  " +
                      "  ████      ████  " +
                      "  ████      ████  " +
                      "  ████      ████  " +
                      "  ████      ████  ")
            csBitmap = getBitmapBytes( 'You don't expect me to make a DIFFERENT piece of beautiful dot matrix art for each language do you?  One is plenty thank you very much.
                      "          ██████              ████████                                                        " +
                      "        ██      ██          ██        ██                                                      " +
                      "      ██          ██      ██            ██                                                    " +
                      "    ██            ██    ██              ██                                                    " +
                      "  ██              ██  ██                ██                                                    " +
                      "  ██              ██  ██              ██                                                      " +
                      "██                ██  ██            ██                                                        " +
                      "██              ██    ██                                                                      " +
                      "██            ██      ██                        ██                                            " +
                      "██        ████          ██                      ██                                            " +
                      "██                        ██                    ██                                            " +
                      "██                          ██                ██                                              " +
                      "██                ██          ████            ██                                              " +
                      "██              ██                ████        ██                                              " +
                      "██              ██      ██            ██      ██████        ████      ██      ████  ████████  " +
                      "██            ██      ██                ██  ██      ██    ██    ██      ██  ██      ██      ██" +
                      "██          ██        ██                ██  ██      ██  ██      ██      ████        ██      ██" +
                      "  ██      ██          ██              ██    ██      ██  ██      ██      ██          ██      ██" +
                      "    ██████              ██          ██      ██      ██  ██    ████    ██            ██    ██  " +
                      "                          ██████████      ██        ████  ████    ██  ██            ██████    " +
                      "                                                                                    ██        " +
                      "                                                                                    ██        " +
                      "                                                                                  ██          " +
                      "                                                                                  ██          " +
                      "                                                                                  ██          ")
#End Region

            'draw some stuff
            device.WriteText(LCDFont.Dimensions_6x12, 0, 0, "A few boldface 'A's")
            device.SetFontSize(LCDFont.User1, 9, 10)
            device.SetCharacterBitmap(LCDFont.User1, "A", aBitmap)
            device.WriteText(LCDFont.User1, 0, 13, "AAAAAA")
            device.DrawLine(5, 40, 35, 30)
            device.DrawRectangle(20, 35, 30, 45, True, False)
            device.Copy(0, 0, 5, 30, 35, 45, 87, 30, False)
            device.WriteBitmap(40, 35, 47, 25, csBitmap)
            device.DrawPixel(64, 32, True)       'this pixel is in the centre Of your screen
            'write the frame buffer to the screen
            device.Flush()

        Catch ex As Exception
            errorBox.addMessage("Error drawing image: " + ex.Message)
        End Try
    End Sub

    Private Sub saveFrameBtn_Click(sender As Object, e As EventArgs) Handles saveFrameBtn.Click
        timer2.Stop()
        Try
            'copy the contents of frame buffer 0 to frame buffer 2
            device.Copy(0, 2, 0, 0, LCD_WIDTH_PIXELS - 1, LCD_HEIGHT_PIXELS - 1, 0, 0, False)

            'store frame buffer 2 in flash memory so it persists through power cycling
            device.SaveFrameBuffer(2)
        Catch ex As Exception
            errorBox.addMessage("Error saving: " + ex.Message)
        End Try
    End Sub

    Private Sub sleepbtn_Click(sender As Object, e As EventArgs) Handles sleepbtn.Click
        timer2.Stop()
        Try
            'put the device to sleep.  useful for preserving backlight life and power saving
            sleepCNT += 1
            device.Sleeping = If(sleepCNT Mod 2 = 0, False, True)
        Catch ex As Exception
            errorBox.addMessage("Error sleeping: " + ex.Message)
        End Try
    End Sub

    Private Sub recallBtn_Click(sender As Object, e As EventArgs) Handles recallBtn.Click
        timer2.Stop()
        Try
            'write stored data from framebuffer 2 to the screen
            device.FrameBuffer = 2
            device.Flush()
        Catch ex As Exception
            errorBox.addMessage("Error recalling frame: " + ex.Message)
        End Try
    End Sub

    Private Sub clearBtn_Click(sender As Object, e As EventArgs) Handles clearBtn.Click
        timer2.Stop()
        Try
            'clear frame buffer 0 and write it to the screen
            device.FrameBuffer = 0
            device.Clear()
            device.Flush()
        Catch ex As Exception
            errorBox.addMessage("Error clearing screen: " + ex.Message)
        End Try
    End Sub

    Private Sub asyncTxtBtn_Click(sender As Object, e As EventArgs) Handles asyncTxtBtn.Click
        timer2.Stop()
        'select the frame buffer to draw to
        device.FrameBuffer = 0
        device.Clear()

        'more readable in the code than a byte array
        Dim funnyface As Byte() = getBitmapBytes(
                "██        ██" +
                "  ██    ██  " +
                "    ████    " +
                "            " +
                "  ██    ██  " +
                "            " +
                "    ████    " +
                "            " +
                "██        ██" +
                "██        ██" +
                "  ██    ██  " +
                "    ████    ")

        Dim space As Byte() = getBitmapBytes(
                "            " +
                "            " +
                "            " +
                "            " +
                "            " +
                "            " +
                "            " +
                "            " +
                "            " +
                "            " +
                "            " +
                "            ")

        Try
            device.SetFontSize(LCDFont.User2, 6, 12)
            device.SetCharacterBitmap(LCDFont.User2, "0", funnyface)
            device.SetCharacterBitmap(LCDFont.User2, " ", space)
            device.WriteText(LCDFont.User2, 0, 35, "0 0 0")

            Dim thing1() As Byte = {1, 0, 1, 0, 1, 0, 1, 0, 1, 0}
            Dim thing2() As Byte = {0, 1, 0, 1, 0, 1, 0, 1, 0, 1}
            Dim space2() As Byte = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
            device.SetFontSize(LCDFont.User1, 1, 10)
            device.SetCharacterBitmap(LCDFont.User1, "1", thing1)
            device.SetCharacterBitmap(LCDFont.User1, "2", thing2)
            device.SetCharacterBitmap(LCDFont.User1, " ", space2)
        Catch ex As Exception
            errorBox.addMessage("Error setting up customs chars: " + ex.Message)
        End Try

        Try
            Dim ret1 As IAsyncResult = device.BeginWriteText(LCDFont.User1, 0, 0, "121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 12121212", Nothing, Nothing)
            Dim ret2 As IAsyncResult = device.BeginWriteText(LCDFont.User1, 0, 10, " 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 1212121", Nothing, Nothing)
            Dim ret3 As IAsyncResult = device.BeginWriteText(LCDFont.User1, 0, 20, "121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 12121212", Nothing, Nothing)

            device.EndWriteText(ret1)
            device.EndWriteText(ret2)
            device.EndWriteText(ret3)
        Catch ex As Exception
            errorBox.addMessage("Async Error writing text: " + ex.Message)
        End Try

        device.BeginWriteText(LCDFont.Dimensions_6x12, 5, 5, "Overwriting!", Nothing, Nothing)
        'write the frame buffer to the screen
        device.Flush()
    End Sub

    Private Sub button1_Click(sender As Object, e As EventArgs) Handles button1.Click
        If (asyncTextThread IsNot Nothing) Then
            asyncTextThreadRun = False
            asyncTextThread.Join()
            asyncTextThread = Nothing
        Else
            asyncTextThread = New Thread(New ThreadStart(AddressOf asyncTextThreadFunction))
            asyncTextThreadRun = True
            asyncTextThread.Start()
        End If
    End Sub

    Private Sub imageBtn_Click(sender As Object, e As EventArgs)
        timer2.Stop()
        Try
            Dim open As OpenFileDialog = New OpenFileDialog()

            open.Filter = "Image Files(*.jpg; *.jpeg; *.gif; *.bmp; *.png)|*.jpg; *.jpeg; *.gif; *.bmp; *.png"
            If open.ShowDialog() = DialogResult.OK Then
                Dim originalImg As Image = Image.FromFile(open.FileName)
                Dim numberOfFrames As Integer = 1
                Try
                    numberOfFrames = originalImg.GetFrameCount(FrameDimension.Time)
                Catch ex As Exception
                End Try
                If numberOfFrames > 1 Then
                    If writeAnimatedGifThread IsNot Nothing And writeAnimatedGifThread.IsAlive Then
                        stop_writeAnimatedGifThread = True
                        writeAnimatedGifThread.Join()
                    End If

                    writeAnimatedGifThread = New Thread(New ThreadStart(AddressOf writeAnimatedGifThreadFunction))
                    image = originalImg
                    stop_writeAnimatedGifThread = False
                    writeAnimatedGifThread.Start()
                Else
                    Dim pic As Bitmap = New Bitmap(originalImg)
                    writeImageGreyscale(pic)
                End If
            End If
        Catch ex As Exception
            errorBox.addMessage("Error loading image: " + ex.Message)
        End Try
    End Sub
End Class



